@@ -11,15 +11,12 @@
 
 #include "customextension.h"
 
-static void registerTypes()
-{
-    qmlRegisterType<CustomExtensionQuickExtension>("io.qt.examples.customextension", 1, 0, "CustomExtension");
-}
-
 int main(int argc, char *argv[])
 {
+    // ShareOpenGLContexts is needed for using the threaded renderer
+    // on Nvidia EGLStreams
+    QCoreApplication::setAttribute(Qt::AA_ShareOpenGLContexts, true);
     QGuiApplication app(argc, argv);
-    registerTypes();
     QQmlApplicationEngine appEngine(QUrl("qrc:///qml/main.qml"));
 
     return app.exec();
