@@ -57,6 +57,8 @@
 #include "qwaylandinputdeviceintegrationfactory_p.h"
 #include "qwaylandwindow_p.h"
 
+#include <QtWaylandClient/private/qwayland-xdg-system-bell-v1.h>
+
 #if QT_CONFIG(accessibility_atspi_bridge)
 #include <QtGui/private/qspiaccessiblebridge_p.h>
 #endif
@@ -123,7 +125,7 @@ bool QWaylandIntegration::hasCapability(QPlatformIntegration::Capability cap) co
     case RasterGLSurface:
         return true;
     case WindowActivation:
-        return false;
+        return true;
     case ScreenWindowGrabbing: // whether QScreen::grabWindow() is supported
         return false;
     default: return QPlatformIntegration::hasCapability(cap);
@@ -529,6 +531,14 @@ void QWaylandIntegration::setApplicationBadge(qint64 number)
 {
     mPlatformServices->setApplicationBadge(number);
 }
+
+void QWaylandIntegration::beep() const
+{
+    if (auto bell = mDisplay->systemBell()) {
+        bell->ring(nullptr);
+    }
+}
+
 }
 
 QT_END_NAMESPACE
