@@ -1,31 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2020 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtWaylandCompositor module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 or (at your option) any later version
-** approved by the KDE Free Qt Foundation. The licenses are as published by
-** the Free Software Foundation and appearing in the file LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2020 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include "qwaylandqttextinputmethod.h"
 #include "qwaylandqttextinputmethod_p.h"
@@ -306,7 +280,7 @@ void QWaylandQtTextInputMethod::sendInputMethodEvent(QInputMethodEvent *event)
                                                  attribute.type,
                                                  attribute.start,
                                                  attribute.length,
-                                                 attribute.value.typeId() == QVariant::Color ? attribute.value.value<QColor>().name() : QString());
+                                                 attribute.value.typeId() == QMetaType::QColor ? attribute.value.value<QColor>().name() : QString());
             break;
         case QInputMethodEvent::Language: // ### What is the type of value? Is it string?
             Q_FALLTHROUGH();
@@ -337,8 +311,13 @@ void QWaylandQtTextInputMethod::sendInputMethodEvent(QInputMethodEvent *event)
                                    event->replacementStart(),
                                    event->replacementLength());
 
-    while (d->waitingForSync)
+    while (d->waitingForSync) {
         d->compositor->processWaylandEvents();
+        // We might get into a situation where the client is waiting for us to
+        // until we confirm the frame is rendered until that he cannot answer
+        // our input events.
+        QCoreApplication::processEvents();
+    }
 
     Qt::InputMethodQueries queries;
     if (d->surroundingText != oldSurroundText)
@@ -370,7 +349,7 @@ void QWaylandQtTextInputMethod::setFocus(QWaylandSurface *surface)
     Q_D(QWaylandQtTextInputMethod);
 
     QWaylandQtTextInputMethodPrivate::Resource *resource = surface != nullptr ? d->resourceMap().value(surface->waylandClient()) : nullptr;
-    if (d->resource == resource)
+    if (d->resource == resource && d->focusedSurface == surface) // same client, same surface
         return;
 
     if (d->resource != nullptr && d->focusedSurface != nullptr) {
@@ -457,3 +436,5 @@ QByteArray QWaylandQtTextInputMethod::interfaceName()
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qwaylandqttextinputmethod.cpp"
