@@ -12,7 +12,7 @@ QT_BEGIN_NAMESPACE
 
 /*!
  * \qmltype XdgOutputManagerV1
- * \instantiates QWaylandXdgOutputManagerV1
+ * \nativetype QWaylandXdgOutputManagerV1
  * \inqmlmodule QtWayland.Compositor.XdgShell
  * \since 5.14
  * \brief Provides an extension for describing outputs in a desktop oriented fashion.
@@ -130,7 +130,6 @@ void QWaylandXdgOutputManagerV1Private::registerXdgOutput(QWaylandOutput *output
 {
     if (!xdgOutputs.contains(output)) {
         xdgOutputs[output] = xdgOutput;
-        QWaylandOutputPrivate::get(output)->xdgOutput = xdgOutput;
     }
 }
 
@@ -139,6 +138,11 @@ void QWaylandXdgOutputManagerV1Private::unregisterXdgOutput(QWaylandOutput *outp
     xdgOutputs.remove(output);
 }
 
+QWaylandXdgOutputV1 *QWaylandXdgOutputManagerV1Private::xdgOutput(QWaylandOutput *output) const
+{
+    return xdgOutputs.value(output);
+}
+
 void QWaylandXdgOutputManagerV1Private::zxdg_output_manager_v1_get_xdg_output(Resource *resource,
                                                                               uint32_t id,
                                                                               wl_resource *outputResource)
@@ -206,7 +210,7 @@ QWaylandXdgOutputV1::~QWaylandXdgOutputV1()
 }
 
 /*!
- * \qmlproperty XdgOutputManagerV1 QtWaylandCompositor::XdgOutputV1::manager
+ * \qmlproperty XdgOutputManagerV1 XdgOutputV1::manager
  * \readonly
  *
  * This property holds the object that manages this XdgOutputV1.
@@ -224,7 +228,7 @@ QWaylandXdgOutputManagerV1 *QWaylandXdgOutputV1::manager() const
 }
 
 /*!
- * \qmlproperty WaylandOutput QtWaylandCompositor::XdgOutputV1::output
+ * \qmlproperty WaylandOutput XdgOutputV1::output
  * \readonly
  *
  * This property holds the WaylandOutput associated with this XdgOutputV1.
@@ -242,7 +246,7 @@ QWaylandOutput *QWaylandXdgOutputV1::output() const
 }
 
 /*!
- * \qmlproperty string QtWaylandCompositor::XdgOutputV1::name
+ * \qmlproperty string XdgOutputV1::name
  *
  * This property holds the name of this output.
  *
@@ -293,7 +297,7 @@ void QWaylandXdgOutputV1::setName(const QString &name)
 }
 
 /*!
- *  \qmlproperty string QtWaylandCompositor::XdgOutputV1::description
+ *  \qmlproperty string XdgOutputV1::description
  *
  *  This property holds the description of this output.
  *
@@ -334,7 +338,7 @@ void QWaylandXdgOutputV1::setDescription(const QString &description)
 }
 
 /*!
- * \qmlproperty point QtWaylandCompositor::XdgOutputV1::logicalPosition
+ * \qmlproperty point XdgOutputV1::logicalPosition
  *
  * This property holds the coordinates of the output within the global compositor space.
  *
@@ -370,7 +374,7 @@ void QWaylandXdgOutputV1::setLogicalPosition(const QPoint &position)
 }
 
 /*!
- * \qmlproperty size QtWaylandCompositor::XdgOutputV1::logicalSize
+ * \qmlproperty size XdgOutputV1::logicalSize
  *
  * This property holds the size of the output in the global compositor space.
  *
@@ -422,7 +426,7 @@ void QWaylandXdgOutputV1::setLogicalSize(const QSize &size)
 }
 
 /*!
- * \qmlproperty rect QtWaylandCompositor::XdgOutputV1::logicalGeometry
+ * \qmlproperty rect XdgOutputV1::logicalGeometry
  * \readonly
  *
  * This property holds the position and size of the output in the global compositor space.
@@ -454,7 +458,6 @@ void QWaylandXdgOutputV1Private::sendLogicalPosition(const QPoint &position)
     const auto values = resourceMap().values();
     for (auto *resource : values)
         send_logical_position(resource->handle, position.x(), position.y());
-    needToSendDone = true;
 }
 
 void QWaylandXdgOutputV1Private::sendLogicalSize(const QSize &size)
@@ -462,19 +465,17 @@ void QWaylandXdgOutputV1Private::sendLogicalSize(const QSize &size)
     const auto values = resourceMap().values();
     for (auto *resource : values)
         send_logical_size(resource->handle, size.width(), size.height());
-    needToSendDone = true;
 }
 
 void QWaylandXdgOutputV1Private::sendDone()
 {
-    if (needToSendDone) {
-        const auto values = resourceMap().values();
-        for (auto *resource : values) {
-            if (resource->version() < 3)
-                send_done(resource->handle);
-        }
-        needToSendDone = false;
+    const auto values = resourceMap().values();
+    for (auto *resource : values) {
+        if (resource->version() < 3)
+            send_done(resource->handle);
     }
+
+    QWaylandOutputPrivate::get(output)->sendDone();
 }
 
 void QWaylandXdgOutputV1Private::setManager(QWaylandXdgOutputManagerV1 *_manager)
